.wp-block-tag-cloud .wp-block-tag-cloud{
  border:none;
  border-radius:inherit;
  margin:0;
  padding:0;
}

.wp-block-tag-cloud__inspector-settings .components-base-control,.wp-block-tag-cloud__inspector-settings .components-base-control:last-child{
  margin-bottom:0;
}