.wp-block-social-links div.block-editor-url-input{
  display:inline-block;
  margin-right:8px;
}

.wp-social-link:hover{
  transform:none;
}

:root :where(.wp-block-social-links),:root :where(.wp-block-social-links.is-style-logos-only .wp-block-social-links__social-placeholder .wp-social-link){
  padding:0;
}

:root :where(.wp-block-social-links__social-placeholder .wp-social-link){
  padding:.25em;
}

:root :where(.wp-block-social-links.is-style-pill-shape .wp-block-social-links__social-placeholder .wp-social-link){
  padding-left:.66667em;
  padding-right:.66667em;
}

.wp-block-social-links__social-placeholder{
  display:flex;
  list-style:none;
  opacity:.8;
}
.wp-block-social-links__social-placeholder>.wp-social-link{
  margin-left:0 !important;
  margin-right:0 !important;
  padding-left:0 !important;
  padding-right:0 !important;
  visibility:hidden;
  width:0 !important;
}
.wp-block-social-links__social-placeholder>.wp-block-social-links__social-placeholder-icons{
  display:flex;
}
.wp-block-social-links__social-placeholder .wp-social-link:before{
  border-radius:50%;
  content:"";
  display:block;
  height:1em;
  width:1em;
}
.is-style-logos-only .wp-block-social-links__social-placeholder .wp-social-link:before{
  background:currentColor;
}

.wp-block-social-links .wp-block-social-links__social-prompt{
  cursor:default;
  font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen-Sans,Ubuntu,Cantarell,Helvetica Neue,sans-serif;
  font-size:13px;
  line-height:24px;
  list-style:none;
  margin-bottom:auto;
  margin-top:auto;
  min-height:24px;
  padding-left:8px;
}

.wp-block.wp-block-social-links.aligncenter,.wp-block[data-align=center]>.wp-block-social-links{
  justify-content:center;
}

.block-editor-block-preview__content .components-button:disabled{
  opacity:1;
}

.wp-social-link.wp-social-link__is-incomplete{
  opacity:.5;
}
@media (prefers-reduced-motion:reduce){
  .wp-social-link.wp-social-link__is-incomplete{
    transition-delay:0s;
    transition-duration:0s;
  }
}

.wp-block-social-links .is-selected .wp-social-link__is-incomplete,.wp-social-link.wp-social-link__is-incomplete:focus,.wp-social-link.wp-social-link__is-incomplete:hover{
  opacity:1;
}

.wp-block-social-links .block-list-appender{
  position:static;
}
.wp-block-social-links .block-list-appender .block-editor-button-block-appender.components-button.components-button{
  padding:6px;
}

.wp-block-social-links.has-small-icon-size .block-editor-button-block-appender.components-button.components-button{
  padding:0;
}
.wp-block-social-links.has-large-icon-size .block-editor-button-block-appender.components-button.components-button{
  padding:14px;
}
.wp-block-social-links.has-huge-icon-size .block-editor-button-block-appender.components-button.components-button{
  padding:23px;
}