.block-editor-block-list__block[data-type="core/paragraph"].has-drop-cap:focus{
  min-height:auto !important;
}

.block-editor-block-list__block[data-empty=true] [data-rich-text-placeholder]{
  opacity:1;
}

.block-editor-block-list__block[data-empty=true]+.block-editor-block-list__block[data-empty=true]:not([data-custom-placeholder=true]) [data-rich-text-placeholder]{
  opacity:0;
}

.block-editor-block-list__block[data-type="core/paragraph"].has-text-align-left[style*="writing-mode: vertical-lr"],.block-editor-block-list__block[data-type="core/paragraph"].has-text-align-right[style*="writing-mode: vertical-rl"]{
  rotate:180deg;
}