:root :where(figure.wp-block-gallery){
  display:block;
}
:root :where(figure.wp-block-gallery)>.blocks-gallery-caption{
  flex:0 0 100%;
}
:root :where(figure.wp-block-gallery)>.blocks-gallery-media-placeholder-wrapper{
  flex-basis:100%;
}
:root :where(figure.wp-block-gallery) .wp-block-image .components-notice.is-error{
  display:block;
}
:root :where(figure.wp-block-gallery) .wp-block-image .components-notice__content{
  margin:4px 0;
}
:root :where(figure.wp-block-gallery) .wp-block-image .components-notice__dismiss{
  left:5px;
  position:absolute;
  top:0;
}
:root :where(figure.wp-block-gallery) .block-editor-media-placeholder.is-appender .components-placeholder__label{
  display:none;
}
:root :where(figure.wp-block-gallery) .block-editor-media-placeholder.is-appender .block-editor-media-placeholder__button{
  margin-bottom:0;
}
:root :where(figure.wp-block-gallery) .block-editor-media-placeholder{
  margin:0;
}
:root :where(figure.wp-block-gallery) .block-editor-media-placeholder .components-placeholder__label{
  display:flex;
}
:root :where(figure.wp-block-gallery) .block-editor-media-placeholder figcaption{
  z-index:2;
}
:root :where(figure.wp-block-gallery) .components-spinner{
  margin-right:-9px;
  margin-top:-9px;
  position:absolute;
  right:50%;
  top:50%;
}
.gallery-settings-buttons .components-button:first-child{
  margin-left:8px;
}

.gallery-image-sizes .components-base-control__label{
  margin-bottom:4px;
}
.gallery-image-sizes .gallery-image-sizes__loading{
  align-items:center;
  color:#757575;
  display:flex;
  font-size:12px;
}
.gallery-image-sizes .components-spinner{
  margin:0 4px 0 8px;
}
.blocks-gallery-item figure:not(.is-selected):focus,.blocks-gallery-item img:focus{
  outline:none;
}
.blocks-gallery-item figure.is-selected:before{
  bottom:0;
  box-shadow:0 0 0 1px #fff inset, 0 0 0 3px var(--wp-admin-theme-color) inset;
  content:"";
  left:0;
  outline:2px solid #0000;
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
  z-index:1;
}
.blocks-gallery-item figure.is-transient img{
  opacity:.3;
}
.blocks-gallery-item .is-selected .block-library-gallery-item__inline-menu{
  display:inline-flex;
}
.blocks-gallery-item .block-editor-media-placeholder{
  height:100%;
  margin:0;
}
.blocks-gallery-item .block-editor-media-placeholder .components-placeholder__label{
  display:flex;
}

.block-library-gallery-item__inline-menu{
  background:#fff;
  border:1px solid #1e1e1e;
  border-radius:2px;
  display:none;
  margin:8px;
  position:absolute;
  top:-2px;
  transition:box-shadow .2s ease-out;
  z-index:20;
}
@media (prefers-reduced-motion:reduce){
  .block-library-gallery-item__inline-menu{
    transition-delay:0s;
    transition-duration:0s;
  }
}
.block-library-gallery-item__inline-menu:hover{
  box-shadow:0 1px 1px #00000008,0 1px 2px #00000005,0 3px 3px #00000005,0 4px 4px #00000003;
}
@media (min-width:600px){
  .columns-7 .block-library-gallery-item__inline-menu,.columns-8 .block-library-gallery-item__inline-menu{
    padding:2px;
  }
}
.block-library-gallery-item__inline-menu .components-button.has-icon:not(:focus){
  border:none;
  box-shadow:none;
}
@media (min-width:600px){
  .columns-7 .block-library-gallery-item__inline-menu .components-button.has-icon,.columns-8 .block-library-gallery-item__inline-menu .components-button.has-icon{
    height:inherit;
    padding:0;
    width:inherit;
  }
}
.block-library-gallery-item__inline-menu.is-left{
  right:-2px;
}
.block-library-gallery-item__inline-menu.is-right{
  left:-2px;
}

.wp-block-gallery ul.blocks-gallery-grid{
  margin:0;
  padding:0;
}

@media (min-width:600px){
  .wp-block-update-gallery-modal{
    max-width:480px;
  }
}

.wp-block-update-gallery-modal-buttons{
  display:flex;
  gap:12px;
  justify-content:flex-end;
}