.wp-block-video{
  box-sizing:border-box;
}
.wp-block-video video{
  vertical-align:middle;
  width:100%;
}
@supports (position:sticky){
  .wp-block-video [poster]{
    object-fit:cover;
  }
}
.wp-block-video.aligncenter{
  text-align:center;
}
.wp-block-video :where(figcaption){
  margin-bottom:1em;
  margin-top:.5em;
}