<?php

// This file was autogenerated by tools/release/sync-stable-blocks.js, do not change manually!
// Returns folder names for static blocks necessary for core blocks registration.
return array(
	'audio',
	'buttons',
	'code',
	'column',
	'columns',
	'details',
	'embed',
	'freeform',
	'group',
	'html',
	'list-item',
	'missing',
	'more',
	'nextpage',
	'paragraph',
	'preformatted',
	'pullquote',
	'quote',
	'separator',
	'social-links',
	'spacer',
	'table',
	'text-columns',
	'verse',
	'video',
);
