.wp-block-button__link{
  background-color:#32373c;
  border-radius:9999px;
  box-shadow:none;
  color:#fff;
  font-size:1.125em;
  padding:calc(.667em + 2px) calc(1.333em + 2px);
  text-decoration:none;
}

.wp-block-file__button{
  background:#32373c;
  color:#fff;
}