:root{
  --wp-admin-theme-color:#007cba;
  --wp-admin-theme-color--rgb:0, 124, 186;
  --wp-admin-theme-color-darker-10:#006ba1;
  --wp-admin-theme-color-darker-10--rgb:0, 107, 161;
  --wp-admin-theme-color-darker-20:#005a87;
  --wp-admin-theme-color-darker-20--rgb:0, 90, 135;
  --wp-admin-border-width-focus:2px;
  --wp-block-synced-color:#7a00df;
  --wp-block-synced-color--rgb:122, 0, 223;
  --wp-bound-block-color:var(--wp-block-synced-color);
}
@media (min-resolution:192dpi){
  :root{
    --wp-admin-border-width-focus:1.5px;
  }
}

.block-editor-format-toolbar__image-popover{
  z-index:159990;
}
.block-editor-format-toolbar__image-popover .block-editor-format-toolbar__image-container-content{
  padding:16px;
  width:260px;
}

.block-editor-format-toolbar__link-container-content{
  align-items:center;
  display:flex;
}

.block-editor-format-toolbar__link-container-value{
  flex-grow:1;
  flex-shrink:1;
  margin:7px;
  max-width:500px;
  min-width:150px;
  overflow:hidden;
  text-overflow:ellipsis;
  white-space:nowrap;
}
.block-editor-format-toolbar__link-container-value.has-invalid-link{
  color:#cc1818;
}

.format-library__inline-color-popover [role=tabpanel]{
  padding:16px;
}

.block-editor-format-toolbar__language-popover .components-popover__content{
  padding:1rem;
  width:auto;
}