:root{
  --wp-admin-theme-color:#007cba;
  --wp-admin-theme-color--rgb:0, 124, 186;
  --wp-admin-theme-color-darker-10:#006ba1;
  --wp-admin-theme-color-darker-10--rgb:0, 107, 161;
  --wp-admin-theme-color-darker-20:#005a87;
  --wp-admin-theme-color-darker-20--rgb:0, 90, 135;
  --wp-admin-border-width-focus:2px;
  --wp-block-synced-color:#7a00df;
  --wp-block-synced-color--rgb:122, 0, 223;
  --wp-bound-block-color:var(--wp-block-synced-color);
}
@media (min-resolution:192dpi){
  :root{
    --wp-admin-border-width-focus:1.5px;
  }
}

.block-editor-block-icon{
  align-items:center;
  display:flex;
  height:24px;
  justify-content:center;
  width:24px;
}
.block-editor-block-icon.has-colors svg{
  fill:currentColor;
}
@media (forced-colors:active){
  .block-editor-block-icon.has-colors svg{
    fill:CanvasText;
  }
}
.block-editor-block-icon svg{
  max-height:24px;
  max-width:24px;
  min-height:20px;
  min-width:20px;
}

.block-editor-block-styles .block-editor-block-list__block{
  margin:0;
}
@keyframes selection-overlay__fade-in-animation{
  0%{
    opacity:0;
  }
  to{
    opacity:.4;
  }
}
:root .block-editor-block-list__layout::selection,:root .has-multi-selection .block-editor-block-list__layout::selection,_::-webkit-full-page-media,_:future{
  background-color:initial;
}
.block-editor-block-list__layout{
  position:relative;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-multi-selected:not(.is-partially-selected) ::selection,.block-editor-block-list__layout .block-editor-block-list__block.is-multi-selected:not(.is-partially-selected)::selection{
  background:#0000;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-multi-selected:not(.is-partially-selected):after{
  animation:selection-overlay__fade-in-animation .1s ease-out;
  animation-fill-mode:forwards;
  background:var(--wp-admin-theme-color);
  bottom:0;
  content:"";
  left:0;
  opacity:.4;
  outline:2px solid #0000;
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
  z-index:1;
}
@media (prefers-reduced-motion:reduce){
  .block-editor-block-list__layout .block-editor-block-list__block.is-multi-selected:not(.is-partially-selected):after{
    animation-delay:0s;
    animation-duration:1ms;
  }
}
.block-editor-block-list__layout .block-editor-block-list__block.is-multi-selected:not(.is-partially-selected).is-highlighted:after{
  outline-color:#0000;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-highlighted,.block-editor-block-list__layout .block-editor-block-list__block.is-highlighted~.is-multi-selected,.block-editor-block-list__layout .block-editor-block-list__block:not([contenteditable=true]):focus,.block-editor-block-list__layout.is-navigate-mode .block-editor-block-list__block.is-selected{
  outline:none;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-highlighted:after,.block-editor-block-list__layout .block-editor-block-list__block.is-highlighted~.is-multi-selected:after,.block-editor-block-list__layout .block-editor-block-list__block:not([contenteditable=true]):focus:after,.block-editor-block-list__layout.is-navigate-mode .block-editor-block-list__block.is-selected:after{
  bottom:0;
  content:"";
  left:0;
  outline-color:var(--wp-admin-theme-color);
  outline-offset:calc(((-1*var(--wp-admin-border-width-focus))/var(--wp-block-editor-iframe-zoom-out-scale, 1))*1);
  outline-style:solid;
  outline-width:calc((var(--wp-admin-border-width-focus)/var(--wp-block-editor-iframe-zoom-out-scale, 1))*1);
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
  z-index:1;
}
.block-editor-block-list__layout .is-block-moving-mode.block-editor-block-list__block.is-selected:after{
  border-radius:2px;
  border-top:4px solid #ccc;
  bottom:auto;
  box-shadow:none;
  content:"";
  left:0;
  pointer-events:none;
  position:absolute;
  right:0;
  top:-14px;
  transition:border-color .1s linear,border-style .1s linear,box-shadow .1s linear;
  z-index:0;
}
.block-editor-block-list__layout .is-block-moving-mode.can-insert-moving-block.block-editor-block-list__block.is-selected:after{
  border-color:var(--wp-admin-theme-color);
}
.has-multi-selection .block-editor-block-list__layout{
  -webkit-user-select:none;
          user-select:none;
}
.block-editor-block-list__layout [class^=components-]{
  -webkit-user-select:text;
          user-select:text;
}

.is-block-moving-mode.block-editor-block-list__block-selection-button{
  font-size:1px;
  height:1px;
  opacity:0;
  padding:0;
}

.block-editor-block-list__layout .block-editor-block-list__block{
  overflow-wrap:break-word;
  pointer-events:auto;
  position:relative;
  -webkit-user-select:text;
          user-select:text;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-editing-disabled{
  pointer-events:none;
  -webkit-user-select:none;
          user-select:none;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-negative-margin.has-child-selected,.block-editor-block-list__layout .block-editor-block-list__block.has-negative-margin.is-selected{
  z-index:20;
}
.block-editor-block-list__layout .block-editor-block-list__block .reusable-block-edit-panel *{
  z-index:1;
}
.block-editor-block-list__layout .block-editor-block-list__block .components-placeholder .components-with-notices-ui{
  margin:-10px 0 12px;
}
.block-editor-block-list__layout .block-editor-block-list__block .components-with-notices-ui{
  margin:0 0 12px;
  width:100%;
}
.block-editor-block-list__layout .block-editor-block-list__block .components-with-notices-ui .components-notice .components-notice__content{
  font-size:13px;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-warning{
  min-height:48px;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-warning>*{
  pointer-events:none;
  -webkit-user-select:none;
          user-select:none;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-warning .block-editor-warning{
  pointer-events:all;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-warning:after{
  background-color:#fff6;
  bottom:0;
  content:"";
  left:0;
  position:absolute;
  right:0;
  top:0;
}
.block-editor-block-list__layout .block-editor-block-list__block.has-warning.is-multi-selected:after{
  background-color:initial;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-reusable>.block-editor-inner-blocks>.block-editor-block-list__layout.has-overlay:after{
  display:none;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-reusable>.block-editor-inner-blocks>.block-editor-block-list__layout.has-overlay .block-editor-block-list__layout.has-overlay:after{
  display:block;
}
.block-editor-block-list__layout .block-editor-block-list__block.is-reusable.has-child-selected:after{
  box-shadow:0 0 0 1px var(--wp-admin-theme-color);
}
.block-editor-block-list__layout .block-editor-block-list__block[data-clear=true]{
  float:none;
}

.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-hovered,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-selected{
  cursor:default;
}
.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-hovered .rich-text,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-hovered.rich-text,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-selected .rich-text,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-selected.rich-text{
  cursor:auto;
}
.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-hovered:not(.is-selected):after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline):not(.rich-text):not([contenteditable=true]).is-selected:after{
  bottom:0;
  content:"";
  left:0;
  outline-color:var(--wp-admin-theme-color);
  outline-offset:calc(((-1*var(--wp-admin-border-width-focus))/var(--wp-block-editor-iframe-zoom-out-scale, 1))*1);
  outline-style:solid;
  outline-width:calc((var(--wp-admin-border-width-focus)/var(--wp-block-editor-iframe-zoom-out-scale, 1))*1);
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
}
.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-reusable.block-editor-block-list__block:not([contenteditable]):focus:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-reusable.is-highlighted:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-reusable.is-hovered:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).is-reusable.is-selected:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).wp-block-template-part.block-editor-block-list__block:not([contenteditable]):focus:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).wp-block-template-part.is-highlighted:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).wp-block-template-part.is-hovered:after,.is-outline-mode .block-editor-block-list__block:not(.remove-outline).wp-block-template-part.is-selected:after{
  outline-color:var(--wp-block-synced-color);
}

@keyframes block-editor-is-editable__animation{
  0%{
    background-color:rgba(var(--wp-admin-theme-color--rgb), .1);
  }
  to{
    background-color:rgba(var(--wp-admin-theme-color--rgb), 0);
  }
}
@keyframes block-editor-is-editable__animation_reduce-motion{
  0%{
    background-color:rgba(var(--wp-admin-theme-color--rgb), .1);
  }
  99%{
    background-color:rgba(var(--wp-admin-theme-color--rgb), .1);
  }
  to{
    background-color:rgba(var(--wp-admin-theme-color--rgb), 0);
  }
}
.is-root-container:not([inert]) .block-editor-block-list__block.is-reusable.is-selected .block-editor-block-list__block.has-editable-outline:after{
  animation-delay:.1s;
  animation-duration:.8s;
  animation-fill-mode:backwards;
  animation-name:block-editor-is-editable__animation;
  animation-timing-function:ease-out;
  bottom:0;
  content:"";
  left:0;
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
}
@media (prefers-reduced-motion:reduce){
  .is-root-container:not([inert]) .block-editor-block-list__block.is-reusable.is-selected .block-editor-block-list__block.has-editable-outline:after{
    animation-delay:0s;
    animation-name:block-editor-is-editable__animation_reduce-motion;
  }
}

.is-focus-mode .block-editor-block-list__block:not(.has-child-selected){
  opacity:.2;
  transition:opacity .1s linear;
}
@media (prefers-reduced-motion:reduce){
  .is-focus-mode .block-editor-block-list__block:not(.has-child-selected){
    transition-delay:0s;
    transition-duration:0s;
  }
}

.is-focus-mode .block-editor-block-list__block.is-content-locked-temporarily-editing-as-blocks.has-child-selected,.is-focus-mode .block-editor-block-list__block.is-content-locked-temporarily-editing-as-blocks.has-child-selected .block-editor-block-list__block,.is-focus-mode .block-editor-block-list__block.is-content-locked.has-child-selected,.is-focus-mode .block-editor-block-list__block.is-content-locked.has-child-selected .block-editor-block-list__block,.is-focus-mode .block-editor-block-list__block:not(.has-child-selected) .block-editor-block-list__block,.is-focus-mode .block-editor-block-list__block:not(.has-child-selected).is-multi-selected,.is-focus-mode .block-editor-block-list__block:not(.has-child-selected).is-selected{
  opacity:1;
}

.wp-block.alignleft,.wp-block.alignright,.wp-block[data-align=left]>*,.wp-block[data-align=right]>*{
  z-index:21;
}

.wp-site-blocks>[data-align=left]{
  float:right;
  margin-left:2em;
}

.wp-site-blocks>[data-align=right]{
  float:left;
  margin-right:2em;
}

.wp-site-blocks>[data-align=center]{
  justify-content:center;
  margin-left:auto;
  margin-right:auto;
}
.block-editor-block-list .block-editor-inserter{
  cursor:move;
  cursor:grab;
  margin:8px;
}

@keyframes block-editor-inserter__toggle__fade-in-animation{
  0%{
    opacity:0;
  }
  to{
    opacity:1;
  }
}
.wp-block .block-list-appender .block-editor-inserter__toggle{
  animation:block-editor-inserter__toggle__fade-in-animation .1s ease;
  animation-fill-mode:forwards;
}
@media (prefers-reduced-motion:reduce){
  .wp-block .block-list-appender .block-editor-inserter__toggle{
    animation-delay:0s;
    animation-duration:1ms;
  }
}

.block-editor-block-list__block:not(.is-selected):not(.has-child-selected) .block-editor-default-block-appender{
  display:none;
}
.block-editor-block-list__block:not(.is-selected):not(.has-child-selected) .block-editor-default-block-appender .block-editor-inserter__toggle{
  opacity:0;
  transform:scale(0);
}

.block-editor-block-list__block .block-editor-block-list__block-html-textarea{
  border:none;
  border-radius:2px;
  box-shadow:inset 0 0 0 1px #1e1e1e;
  display:block;
  font-family:Menlo,Consolas,monaco,monospace;
  font-size:15px;
  line-height:1.5;
  margin:0;
  outline:none;
  overflow:hidden;
  padding:12px;
  resize:none;
  transition:padding .2s linear;
  width:100%;
}
@media (prefers-reduced-motion:reduce){
  .block-editor-block-list__block .block-editor-block-list__block-html-textarea{
    transition-delay:0s;
    transition-duration:0s;
  }
}
.block-editor-block-list__block .block-editor-block-list__block-html-textarea:focus{
  box-shadow:inset 0 0 0 var(--wp-admin-border-width-focus) var(--wp-admin-theme-color);
}
.block-editor-block-list__block .block-editor-warning{
  position:relative;
  z-index:5;
}
.block-editor-block-list__block .block-editor-warning.block-editor-block-list__block-crash-warning{
  margin-bottom:auto;
}

.block-editor-block-list__zoom-out-separator{
  align-items:center;
  background:#ddd;
  color:#000;
  display:flex;
  font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen-Sans,Ubuntu,Cantarell,Helvetica Neue,sans-serif;
  font-size:13px;
  font-weight:400;
  justify-content:center;
  margin-left:-1px;
  margin-right:-1px;
  overflow:hidden;
  transition:background-color .3s ease;
}
.is-zoomed-out .block-editor-block-list__zoom-out-separator{
  font-size:calc(13px/var(--wp-block-editor-iframe-zoom-out-scale));
}
.block-editor-block-list__zoom-out-separator.is-dragged-over{
  background:#ccc;
}

.block-editor-block-list__layout.is-root-container.has-global-padding>.block-editor-block-list__zoom-out-separator,.has-global-padding>.block-editor-block-list__zoom-out-separator{
  margin:0 calc(var(--wp--style--root--padding-left)*-1 - 1px) 0 calc(var(--wp--style--root--padding-right)*-1 - 1px) !important;
  max-width:none;
}

.is-vertical .block-list-appender{
  margin-left:auto;
  margin-right:12px;
  margin-top:12px;
  width:24px;
}

.block-list-appender>.block-editor-inserter{
  display:block;
}

.block-editor-block-list__block:not(.is-selected):not(.has-child-selected):not(.block-editor-block-list__layout) .block-editor-block-list__layout>.block-list-appender .block-list-appender__toggle{
  opacity:0;
  transform:scale(0);
}

.block-editor-block-list__block.has-block-overlay{
  cursor:default;
}
.block-editor-block-list__block.has-block-overlay .block-editor-block-list__block{
  pointer-events:none;
}
.block-editor-block-list__block.has-block-overlay .block-editor-block-list__block.has-block-overlay:before{
  left:0;
  right:0;
  width:auto;
}

.block-editor-block-list__layout .is-dragging{
  background-color:currentColor !important;
  border-radius:2px !important;
  opacity:.05 !important;
  pointer-events:none !important;
}
.block-editor-block-list__layout .is-dragging::selection{
  background:#0000 !important;
}
.block-editor-block-list__layout .is-dragging:after{
  content:none !important;
}

.block-editor-block-preview__content-iframe .block-list-appender{
  display:none;
}

.block-editor-block-preview__live-content *{
  pointer-events:none;
}
.block-editor-block-preview__live-content .block-list-appender{
  display:none;
}
.block-editor-block-preview__live-content .components-button:disabled{
  opacity:1;
}
.block-editor-block-preview__live-content .block-editor-block-list__block[data-empty=true],.block-editor-block-preview__live-content .components-placeholder{
  display:none;
}

.block-editor-block-variation-picker__skip,.block-editor-block-variation-picker__variations,.wp-block-group-placeholder__variations{
  display:flex;
  flex-direction:row;
  flex-wrap:wrap;
  font-size:12px;
  gap:8px;
  justify-content:flex-start;
  list-style:none;
  margin:0;
  padding:0;
  width:100%;
}
.block-editor-block-variation-picker__skip svg,.block-editor-block-variation-picker__variations svg,.wp-block-group-placeholder__variations svg{
  fill:#949494 !important;
}
.block-editor-block-variation-picker__skip .components-button,.block-editor-block-variation-picker__variations .components-button,.wp-block-group-placeholder__variations .components-button{
  padding:4px;
}
.block-editor-block-variation-picker__skip .components-button:hover,.block-editor-block-variation-picker__variations .components-button:hover,.wp-block-group-placeholder__variations .components-button:hover{
  background:none !important;
}
.block-editor-block-variation-picker__skip .components-button:hover svg,.block-editor-block-variation-picker__variations .components-button:hover svg,.wp-block-group-placeholder__variations .components-button:hover svg{
  fill:var(--wp-admin-theme-color) !important;
}
.block-editor-block-variation-picker__skip>li,.block-editor-block-variation-picker__variations>li,.wp-block-group-placeholder__variations>li{
  align-items:center;
  display:flex;
  flex-direction:column;
  gap:4px;
  width:auto;
}

.block-editor-button-block-appender{
  align-items:center;
  box-shadow:inset 0 0 0 1px #1e1e1e;
  color:#1e1e1e;
  display:flex;
  flex-direction:column;
  height:auto;
  justify-content:center;
  width:100%;
}
.is-dark-theme .block-editor-button-block-appender{
  box-shadow:inset 0 0 0 1px #ffffffa6;
  color:#ffffffa6;
}
.block-editor-button-block-appender:hover{
  box-shadow:inset 0 0 0 1px var(--wp-admin-theme-color);
  color:var(--wp-admin-theme-color);
}
.block-editor-button-block-appender:focus{
  box-shadow:inset 0 0 0 2px var(--wp-admin-theme-color);
}
.block-editor-button-block-appender:active{
  color:#000;
}

.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child{
  pointer-events:none;
}
.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child:after,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child:after,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child:after,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child:after{
  border:1px dashed;
  bottom:0;
  content:"";
  left:0;
  pointer-events:none;
  position:absolute;
  right:0;
  top:0;
}
.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child .block-editor-inserter,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child .block-editor-inserter,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child .block-editor-inserter,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child .block-editor-inserter{
  opacity:0;
}
.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child .block-editor-inserter:focus-within,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child .block-editor-inserter:focus-within,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child .block-editor-inserter:focus-within,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child .block-editor-inserter:focus-within{
  opacity:1;
}
.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child.is-drag-over:after,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child.is-drag-over:after,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child.is-drag-over:after,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child.is-drag-over:after{
  border:none;
}
.block-editor-block-list__block:not(.is-selected)>.is-layout-constrained.wp-block-group__inner-container>.block-list-appender:only-child.is-drag-over .block-editor-inserter,.block-editor-block-list__block:not(.is-selected)>.is-layout-flow.wp-block-group__inner-container>.block-list-appender:only-child.is-drag-over .block-editor-inserter,.is-layout-constrained.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child.is-drag-over .block-editor-inserter,.is-layout-flow.block-editor-block-list__block:not(.is-selected)>.block-list-appender:only-child.is-drag-over .block-editor-inserter{
  visibility:visible;
}
.block-editor-block-list__block:not(.is-selected)>.block-editor-block-list__block>.block-list-appender:only-child:after{
  border:none;
}
.block-list-appender:only-child.is-drag-over .block-editor-button-block-appender{
  background-color:var(--wp-admin-theme-color);
  box-shadow:inset 0 0 0 1px #ffffffa6;
  color:#ffffffa6;
  transition:background-color .2s ease-in-out;
}
@media (prefers-reduced-motion:reduce){
  .block-list-appender:only-child.is-drag-over .block-editor-button-block-appender{
    transition:none;
  }
}
.block-editor-default-block-appender{
  clear:both;
  margin-left:auto;
  margin-right:auto;
  position:relative;
}
.block-editor-default-block-appender[data-root-client-id=""] .block-editor-default-block-appender__content:hover{
  outline:1px solid #0000;
}
.block-editor-default-block-appender .block-editor-default-block-appender__content{
  margin-block-end:0;
  margin-block-start:0;
  opacity:.62;
}
.block-editor-default-block-appender .components-drop-zone__content-icon{
  display:none;
}
.block-editor-default-block-appender .block-editor-inserter__toggle.components-button.has-icon{
  background:#1e1e1e;
  color:#fff;
  height:24px;
  min-width:24px;
  padding:0;
}
.block-editor-default-block-appender .block-editor-inserter__toggle.components-button.has-icon:hover{
  background:var(--wp-admin-theme-color);
  color:#fff;
}

.block-editor-block-list__empty-block-inserter.block-editor-block-list__empty-block-inserter,.block-editor-default-block-appender .block-editor-inserter{
  left:0;
  line-height:0;
  position:absolute;
  top:0;
}
.block-editor-block-list__empty-block-inserter.block-editor-block-list__empty-block-inserter:disabled,.block-editor-default-block-appender .block-editor-inserter:disabled{
  display:none;
}
.block-editor-block-list__block .block-list-appender{
  bottom:0;
  left:0;
  list-style:none;
  padding:0;
  position:absolute;
  z-index:2;
}
.block-editor-block-list__block .block-list-appender.block-list-appender{
  line-height:0;
  margin:0;
}
.block-editor-block-list__block .block-list-appender .block-editor-default-block-appender{
  height:24px;
}
.block-editor-block-list__block .block-list-appender .block-editor-inserter__toggle.components-button.has-icon,.block-editor-block-list__block .block-list-appender .block-list-appender__toggle{
  background:#1e1e1e;
  box-shadow:none;
  color:#fff;
  display:none;
  flex-direction:row;
  height:24px;
  min-width:24px;
  padding:0 !important;
  width:24px;
}
.block-editor-block-list__block .block-list-appender .block-editor-inserter__toggle.components-button.has-icon:hover,.block-editor-block-list__block .block-list-appender .block-list-appender__toggle:hover{
  background:var(--wp-admin-theme-color);
  color:#fff;
}
.block-editor-block-list__block .block-list-appender .block-editor-default-block-appender__content{
  display:none;
}
.block-editor-block-list__block .block-list-appender:only-child{
  align-self:center;
  left:auto;
  line-height:inherit;
  list-style:none;
  position:relative;
}
.block-editor-block-list__block .block-list-appender:only-child .block-editor-default-block-appender__content{
  display:block;
}

.block-editor-block-list__block.is-selected .block-editor-block-list__layout>.block-list-appender .block-editor-inserter__toggle.components-button.has-icon,.block-editor-block-list__block.is-selected .block-editor-block-list__layout>.block-list-appender .block-list-appender__toggle,.block-editor-block-list__block.is-selected>.block-list-appender .block-editor-inserter__toggle.components-button.has-icon,.block-editor-block-list__block.is-selected>.block-list-appender .block-list-appender__toggle{
  display:flex;
}

.block-editor-default-block-appender__content{
  cursor:text;
}

.block-editor-iframe__body{
  position:relative;
}

.block-editor-iframe__html{
  transform-origin:top center;
  transition:all .4s cubic-bezier(.46, .03, .52, .96),transform 0s,scale 0s,padding 0s,translate 0s;
}
@media (prefers-reduced-motion:reduce){
  .block-editor-iframe__html{
    transition-delay:0s;
    transition-duration:0s;
  }
}
.block-editor-iframe__html.zoom-out-animation{
  bottom:0;
  left:0;
  overflow-y:scroll;
  position:fixed;
  right:0;
  top:calc(var(--wp-block-editor-iframe-zoom-out-scroll-top, 0)*-1);
  transition:all .4s cubic-bezier(.46, .03, .52, .96),transform 0s,top 0s,bottom 0s,left 0s,right 0s;
  translate:0 calc(var(--wp-block-editor-iframe-zoom-out-scroll-top, 0) - var(--wp-block-editor-iframe-zoom-out-scroll-top-next, 0));
}
@media (prefers-reduced-motion:reduce){
  .block-editor-iframe__html.zoom-out-animation{
    transition-delay:0s;
    transition-duration:0s;
  }
}

.block-editor-iframe__html.is-zoomed-out{
  background-color:#ddd;
  margin-bottom:calc(var(--wp-block-editor-iframe-zoom-out-content-height)*(1 - var(--wp-block-editor-iframe-zoom-out-scale))*-1 + var(--wp-block-editor-iframe-zoom-out-frame-size)*2/var(--wp-block-editor-iframe-zoom-out-scale)*-1 + -2px);
  padding-bottom:calc(var(--wp-block-editor-iframe-zoom-out-frame-size)/var(--wp-block-editor-iframe-zoom-out-scale));
  padding-top:calc(var(--wp-block-editor-iframe-zoom-out-frame-size)/var(--wp-block-editor-iframe-zoom-out-scale));
  scale:var(--wp-block-editor-iframe-zoom-out-scale);
  transform:translateX(calc(((var(--wp-block-editor-iframe-zoom-out-scale-container-width) - var(--wp-block-editor-iframe-zoom-out-container-width, 100vw))/2/var(--wp-block-editor-iframe-zoom-out-scale))*-1));
}
.block-editor-iframe__html.is-zoomed-out body{
  min-height:calc((var(--wp-block-editor-iframe-zoom-out-inner-height) - 2*var(--wp-block-editor-iframe-zoom-out-frame-size)/var(--wp-block-editor-iframe-zoom-out-scale))/var(--wp-block-editor-iframe-zoom-out-scale));
}
.block-editor-iframe__html.is-zoomed-out body>.is-root-container:not(.wp-block-post-content){
  display:flex;
  flex:1;
  flex-direction:column;
  height:100%;
}
.block-editor-iframe__html.is-zoomed-out body>.is-root-container:not(.wp-block-post-content)>main{
  flex:1;
}

.block-editor-block-list__layout.has-overlay:after{
  bottom:0;
  content:"";
  left:0;
  position:absolute;
  right:0;
  top:0;
  z-index:60;
}

.block-editor-media-placeholder__url-input-form{
  min-width:260px;
}
@media (min-width:600px){
  .block-editor-media-placeholder__url-input-form{
    width:300px;
  }
}

.block-editor-media-placeholder__cancel-button.is-link{
  display:block;
  margin:1em;
}

.block-editor-media-placeholder.is-appender{
  min-height:0;
}
.block-editor-media-placeholder.is-appender:hover{
  box-shadow:0 0 0 1px var(--wp-admin-theme-color);
  cursor:pointer;
}

.block-editor-plain-text{
  border:none;
  box-shadow:none;
  color:inherit;
  font-family:inherit;
  font-size:inherit;
  line-height:inherit;
  margin:0;
  padding:0;
  width:100%;
}

.rich-text [data-rich-text-placeholder]{
  pointer-events:none;
}
.rich-text [data-rich-text-placeholder]:after{
  content:attr(data-rich-text-placeholder);
  opacity:.62;
}
.rich-text:focus{
  outline:none;
}
.rich-text:focus [data-rich-text-format-boundary]{
  border-radius:2px;
}

.block-editor-rich-text__editable>p:first-child{
  margin-top:0;
}

figcaption.block-editor-rich-text__editable [data-rich-text-placeholder]:before{
  opacity:.8;
}

[data-rich-text-script]{
  display:inline;
}
[data-rich-text-script]:before{
  background:#ff0;
  content:"</>";
}

.block-editor-warning{
  align-items:center;
  background-color:#fff;
  border:1px solid #1e1e1e;
  border-radius:2px;
  display:flex;
  flex-wrap:wrap;
  padding:1em;
}
.block-editor-warning,.block-editor-warning .block-editor-warning__message{
  font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen-Sans,Ubuntu,Cantarell,Helvetica Neue,sans-serif;
}
.block-editor-warning .block-editor-warning__message{
  color:#1e1e1e;
  font-size:13px;
  line-height:1.4;
  margin:0;
}
.block-editor-warning p.block-editor-warning__message.block-editor-warning__message{
  min-height:auto;
}
.block-editor-warning .block-editor-warning__contents{
  align-items:baseline;
  display:flex;
  flex-direction:row;
  flex-wrap:wrap;
  gap:12px;
  justify-content:space-between;
  width:100%;
}
.block-editor-warning .block-editor-warning__actions{
  align-items:center;
  display:flex;
  gap:8px;
}

.components-popover.block-editor-warning__dropdown{
  z-index:99998;
}

body.admin-color-light{
  --wp-admin-theme-color:#0085ba;
  --wp-admin-theme-color--rgb:0, 133, 186;
  --wp-admin-theme-color-darker-10:#0073a1;
  --wp-admin-theme-color-darker-10--rgb:0, 115, 161;
  --wp-admin-theme-color-darker-20:#006187;
  --wp-admin-theme-color-darker-20--rgb:0, 97, 135;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-light{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-modern{
  --wp-admin-theme-color:#3858e9;
  --wp-admin-theme-color--rgb:56, 88, 233;
  --wp-admin-theme-color-darker-10:#2145e6;
  --wp-admin-theme-color-darker-10--rgb:33, 69, 230;
  --wp-admin-theme-color-darker-20:#183ad6;
  --wp-admin-theme-color-darker-20--rgb:24, 58, 214;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-modern{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-blue{
  --wp-admin-theme-color:#096484;
  --wp-admin-theme-color--rgb:9, 100, 132;
  --wp-admin-theme-color-darker-10:#07526c;
  --wp-admin-theme-color-darker-10--rgb:7, 82, 108;
  --wp-admin-theme-color-darker-20:#064054;
  --wp-admin-theme-color-darker-20--rgb:6, 64, 84;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-blue{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-coffee{
  --wp-admin-theme-color:#46403c;
  --wp-admin-theme-color--rgb:70, 64, 60;
  --wp-admin-theme-color-darker-10:#383330;
  --wp-admin-theme-color-darker-10--rgb:56, 51, 48;
  --wp-admin-theme-color-darker-20:#2b2724;
  --wp-admin-theme-color-darker-20--rgb:43, 39, 36;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-coffee{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-ectoplasm{
  --wp-admin-theme-color:#523f6d;
  --wp-admin-theme-color--rgb:82, 63, 109;
  --wp-admin-theme-color-darker-10:#46365d;
  --wp-admin-theme-color-darker-10--rgb:70, 54, 93;
  --wp-admin-theme-color-darker-20:#3a2c4d;
  --wp-admin-theme-color-darker-20--rgb:58, 44, 77;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-ectoplasm{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-midnight{
  --wp-admin-theme-color:#e14d43;
  --wp-admin-theme-color--rgb:225, 77, 67;
  --wp-admin-theme-color-darker-10:#dd382d;
  --wp-admin-theme-color-darker-10--rgb:221, 56, 45;
  --wp-admin-theme-color-darker-20:#d02c21;
  --wp-admin-theme-color-darker-20--rgb:208, 44, 33;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-midnight{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-ocean{
  --wp-admin-theme-color:#627c83;
  --wp-admin-theme-color--rgb:98, 124, 131;
  --wp-admin-theme-color-darker-10:#576e74;
  --wp-admin-theme-color-darker-10--rgb:87, 110, 116;
  --wp-admin-theme-color-darker-20:#4c6066;
  --wp-admin-theme-color-darker-20--rgb:76, 96, 102;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-ocean{
    --wp-admin-border-width-focus:1.5px;
  }
}

body.admin-color-sunrise{
  --wp-admin-theme-color:#dd823b;
  --wp-admin-theme-color--rgb:221, 130, 59;
  --wp-admin-theme-color-darker-10:#d97426;
  --wp-admin-theme-color-darker-10--rgb:217, 116, 38;
  --wp-admin-theme-color-darker-20:#c36922;
  --wp-admin-theme-color-darker-20--rgb:195, 105, 34;
  --wp-admin-border-width-focus:2px;
}
@media (min-resolution:192dpi){
  body.admin-color-sunrise{
    --wp-admin-border-width-focus:1.5px;
  }
}