.wp-block-site-logo.aligncenter>div,.wp-block[data-align=center]>.wp-block-site-logo{
  display:table;
  margin-left:auto;
  margin-right:auto;
}

.wp-block-site-logo a{
  pointer-events:none;
}
.wp-block-site-logo .custom-logo-link{
  cursor:inherit;
}
.wp-block-site-logo .custom-logo-link:focus{
  box-shadow:none;
}
.wp-block-site-logo img{
  display:block;
  height:auto;
  max-width:100%;
}
.wp-block-site-logo.is-transient{
  position:relative;
}
.wp-block-site-logo.is-transient img{
  opacity:.3;
}
.wp-block-site-logo.is-transient .components-spinner{
  left:50%;
  margin:0;
  position:absolute;
  top:50%;
  transform:translate(-50%, -50%);
}

.wp-block-site-logo.wp-block-site-logo.is-default-size .components-placeholder{
  height:60px;
  width:60px;
}
.wp-block-site-logo.wp-block-site-logo .components-resizable-box__container,.wp-block-site-logo.wp-block-site-logo>div{
  border-radius:inherit;
}
.wp-block-site-logo.wp-block-site-logo .components-placeholder{
  align-items:center;
  border-radius:inherit;
  display:flex;
  height:100%;
  justify-content:center;
  min-height:48px;
  min-width:48px;
  padding:0;
  width:100%;
}
.wp-block-site-logo.wp-block-site-logo .components-placeholder .components-drop-zone__content-text,.wp-block-site-logo.wp-block-site-logo .components-placeholder .components-form-file-upload{
  display:none;
}
.wp-block-site-logo.wp-block-site-logo .components-placeholder .components-button.components-button{
  align-items:center;
  background:var(--wp-admin-theme-color);
  border-color:var(--wp-admin-theme-color);
  border-radius:50%;
  border-style:solid;
  color:#fff;
  display:flex;
  height:48px;
  justify-content:center;
  margin:auto;
  padding:0;
  position:relative;
  width:48px;
}
.wp-block-site-logo.wp-block-site-logo .components-placeholder .components-button.components-button>svg{
  color:inherit;
}

.block-library-site-logo__inspector-upload-container{
  position:relative;
}
.block-library-site-logo__inspector-upload-container .components-drop-zone__content-icon{
  display:none;
}

.block-library-site-logo__inspector-media-replace-container button.components-button,.block-library-site-logo__inspector-upload-container button.components-button{
  box-shadow:inset 0 0 0 1px #ccc;
  color:#1e1e1e;
  display:block;
  height:40px;
  width:100%;
}
.block-library-site-logo__inspector-media-replace-container button.components-button:hover,.block-library-site-logo__inspector-upload-container button.components-button:hover{
  color:var(--wp-admin-theme-color);
}
.block-library-site-logo__inspector-media-replace-container button.components-button:focus,.block-library-site-logo__inspector-upload-container button.components-button:focus{
  box-shadow:inset 0 0 0 var(--wp-admin-border-width-focus) var(--wp-admin-theme-color);
}
.block-library-site-logo__inspector-media-replace-container .block-library-site-logo__inspector-media-replace-title,.block-library-site-logo__inspector-upload-container .block-library-site-logo__inspector-media-replace-title{
  text-align:start;
  text-align-last:center;
  white-space:normal;
  word-break:break-all;
}

.block-library-site-logo__inspector-media-replace-container .components-dropdown{
  display:block;
}
.block-library-site-logo__inspector-media-replace-container img{
  aspect-ratio:1;
  border-radius:50% !important;
  box-shadow:inset 0 0 0 1px #0003;
  min-width:20px;
  width:20px;
}
.block-library-site-logo__inspector-media-replace-container .block-library-site-logo__inspector-readonly-logo-preview{
  display:flex;
  height:40px;
  padding:6px 12px;
}