ul.wp-block-rss{
  list-style:none;
  padding:0;
}
ul.wp-block-rss.wp-block-rss{
  box-sizing:border-box;
}
ul.wp-block-rss.alignleft{
  margin-right:2em;
}
ul.wp-block-rss.alignright{
  margin-left:2em;
}
ul.wp-block-rss.is-grid{
  display:flex;
  flex-wrap:wrap;
  list-style:none;
  padding:0;
}
ul.wp-block-rss.is-grid li{
  margin:0 0 1em 1em;
  width:100%;
}
@media (min-width:600px){
  ul.wp-block-rss.columns-2 li{
    width:calc(50% - 1em);
  }
  ul.wp-block-rss.columns-3 li{
    width:calc(33.33333% - 1em);
  }
  ul.wp-block-rss.columns-4 li{
    width:calc(25% - 1em);
  }
  ul.wp-block-rss.columns-5 li{
    width:calc(20% - 1em);
  }
  ul.wp-block-rss.columns-6 li{
    width:calc(16.66667% - 1em);
  }
}

.wp-block-rss__item-author,.wp-block-rss__item-publish-date{
  display:block;
  font-size:.8125em;
}